/******************************************************************************
 Semaphore Example Program
 This program will create 2 tasks and a semaphore. The UserMain task will
 block for a semaphore posted from MyTask. 
 
 The output for this example is displayed through the serial debug port, which
 can be viewed with the MTTTY program.
 
*****************************************************************************/
#include <predef.h>
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <ucos.h>
#include <serial.h>
#include <dhcpclient.h>
#include <netinterface.h>

//----- function prototypes -----
extern "C"
{
   void UserMain( void *pd );
}

//----- Global Variables
const char * AppName = "Semaphore Example";
DWORD MyTaskStack[USER_TASK_STK_SIZE];
OS_SEM MySemaphore;


/*-------------------------------------------------------------------
 MyTask
 This task will post to a semaphore every 3 seconds
 -------------------------------------------------------------------*/
void MyTask( void *pdata )
{
   while ( 1 )
   {
      OSTimeDly( TICKS_PER_SECOND * 3 );
      iprintf( "      MyTask: Posted to Semaphore\r\n" );
      OSSemPost( &MySemaphore );
   }
}


/*-------------------------------------------------------------------
 UserMain
 -------------------------------------------------------------------*/
void UserMain( void *pd )
{
   InitializeStack();
   int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf( "\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
      iprintf("Static IP Address: ");
   ShowIP(ib->netIP); iprintf("\r\n");     

   OSChangePrio( MAIN_PRIO );
   EnableAutoUpdate();

   OSSemInit( &MySemaphore, 0 );

   if ( OSTaskCreate( MyTask,
                      NULL,
                      ( void * ) &MyTaskStack[USER_TASK_STK_SIZE],
                      ( void * ) MyTaskStack,
                      MAIN_PRIO + 1 ) != OS_NO_ERR )
   {
      iprintf( "*** Error creating task\r\n" );
   }

   // The while loop will pend on MySemaphore and display a message
   // when a post is detected.
   while ( 1 )
   {
      iprintf( ">>> UserMain: Pending on Semaphore from MyTask\r\n" );
      OSSemPend( &MySemaphore, 0 );
      iprintf( "<<< UserMain: Semaphore was posted from MyTask!\r\n\r\n" );
   }
}


